COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1991 -- All Rights Reserved

PROJECT:	PC GEOS
MODULE:		
FILE:		pserial.asm

AUTHOR:		jimmy lefkowitz

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/12/93		Initial version.

DESCRIPTION:
	code for the PCMCIA Serial Driver

	$Id: pserial.asm,v 1.1.1.14 93/08/12 19:28:39 brianc Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

_PSerialDriver = 1

include pserial.def

include pserialStrings.asm

include pcmConstant.def
include pcmEnumCIS.asm

include Internal/heapInt.def

.ioenable		; tell esp to let us do io

idata	segment

DefPFunction	macro	routine, constant
.assert ($-pfuncs) eq constant*2, <Routine for constant in the wrong slot>
.assert (type routine eq far)
		fptr	routine
		endm

pfuncs		label	fptr.far
DefPFunction PSerialDoNothing,			DR_INIT
DefPFunction PSerialDoNothing,			DR_EXIT
DefPFunction PSerialDoNothing,			DR_SUSPEND
DefPFunction PSerialDoNothing,			DR_UNSUSPEND
DefPFunction PSerialTestDevice,			DRE_TEST_DEVICE
DefPFunction PSerialSetDevice,			DRE_SET_DEVICE
DefPFunction PSerialConfirmRemoval,		DR_PCMCIA_CONFIRM_REMOVAL
DefPFunction PSerialCardReInserted,		DR_PCMCIA_CARD_REINSERTED
DefPFunction PSerialDoNothing,			DR_PCMCIA_POWER_DOWN
DefPFunction PSerialDoNothing,			DR_PCMCIA_POWER_UP
DefPFunction PSerialDeviceTurnedOn,		DR_PCMCIA_DEVICE_TURNED_ON
DefPFunction PSerialDoNothing,			DR_PCMCIA_DEVICE_TURNED_OFF
	.assert ($-pfuncs) eq PCMCIAFunction*2



idata	ends

EnumData	struct
    ED_tupleBuf		byte	PCM_MAX_TUPLE_SIZE dup(?)
    ED_configAddress	dword
    ED_configIndex	byte
    ED_baseAddress	word
    ED_registerMask	word
EnumData	ends

udata	segment

baudRate	word
lineControl	byte

configAddress	dword		; save the address of the port
configIndex	byte	0	; value to put in configAddress
baseAddress	word	0	; address to put port at
registerMask	word	0	; mask for first 16 config registers

udata	ends

Resident segment resource


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialStrategy
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	PCMCIA Serial driver strategy routine

CALLED BY:	EXTERNAL

PASS:		di	- DR_PSERIAL?  command code.  Operation & other data
		passed, returned depends on command.  Current operations are:


	-----------------------------------------------------------
	PASS:	di	- DRE_TEST_DEVICE
		dx:si	- pointer to null-terminated device name string


	RETURN: ax	- DriverPresent code


	-----------------------------------------------------------
	PASS:	di	- DRE_SET_DEVICE
		dx:si	- pointer to null-terminated device name string

	RETURN: nothing

	-----------------------------------------------------------

DESTROYED:	Depends on function, though best to assume all registers
		that are not returning a value
		(Segment registers are preserved)
		Only BP, DS, ES are preserved.

PSEUDO CODE/STRATEGY:
		


KNOWN BUGS/SIDE EFFECTS/IDEAS:
		none

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Adam	3/10/89		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialStrategy	proc	far
		push	bp, ds, es
		mov	bp, dgroup
		mov	ds, bp
		mov	es, bp


		cmp	di, PCMCIAFunction
		ja	unknown		; yup

		;
		; Call the handler...
		;
		mov	ss:[TPD_dataBX], bx
		shl	di
		movdw	bxax, ds:[pfuncs][di]
		call	ProcCallFixedOrMovable

NEC <done:								>
		pop	bp, ds, es
		ret

unknown:
EC <		ERROR	COMMAND_UNKNOWN					>
NEC <		stc			; error -- undefined command	>
NEC <		jmp	done						>
PSerialStrategy	endp


Init	segment resource

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialTestDevice
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	test for the device

CALLED BY:	GLOBAL

PASS:		ds	= dgroup (from strategy)

RETURN:		carry set if device not present

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/12/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialTestDevice	proc	far
enumData	local	EnumData
		uses	bx, cx, dx, di, si
		.enter

	; look through the CIS for a suitable configuration entry.

		mov	ds:[baseAddress], 0
		mov	ss:[enumData].ED_configIndex, 0
		mov	ss:[enumData].ED_baseAddress, 0
		segmov	es, ss
		lea	di, ss:[enumData]
			CheckHack <offset ED_tupleBuf eq 0>
		mov	cx, cs
		mov	dx, offset PSerialCISEnumCallback
		call	PCMEnumCIS

	; transfer what we found to dgroup

		movdw	ds:[configAddress], ss:[enumData].ED_configAddress, ax
		mov	al, ss:[enumData].ED_configIndex
		mov	ds:[configIndex], al
		mov	ax, ss:[enumData].ED_baseAddress
		mov	ds:[baseAddress], ax
		mov	ax, ss:[enumData].ED_registerMask
		mov	ds:[registerMask], ax

	; so the set device routine know that we are only testing

		mov	di, DRE_TEST_DEVICE
		call	PSerialSetDevice
		jc	bad

	; this code take from SerialCheckExists
		mov	dx, di
		add	dx, offset SP_iid
		in	al, dx
		test	al, not mask SerialIID	; these bits should never be on
		jnz	bad
		add	dx, offset SP_status - offset SP_iid
		in	al, dx
		cmp	al, mask SS_TSRE or mask SS_THRE
		jne	bad
		mov	ax, DP_PRESENT
done:
		clc
		.leave
		ret
bad:
		mov	ax, DP_NOT_PRESENT
		jmp	done
PSerialTestDevice	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialDeviceTurnedOn
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Sets up the card for use by the serial driver.

CALLED BY:	GLOBAL
PASS:		ds - dgroup (from strategy routine)
RETURN:		nada
DESTROYED:	nada
 
PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	atw	7/19/93   	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PSerialDeviceTurnedOn	proc	far
	.enter

;	The power to the card may have been turned off since the last
;	DRE_SET_DEVICE call, so re-configure the device for serial use.

	mov	di, DRE_TEST_DEVICE
	call	PSerialSetDevice

	.leave
	ret
PSerialDeviceTurnedOn	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialSetDevice
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	init the PCMCIA card

CALLED BY:	GLOBAL

PASS:		ds	= dgroup (from strategy)
		di = either DRE_SET_DEVICE or DRE_TEST_DEVICE
			depending on where this is being called from

RETURN:		carry set if no device
		di = base address for device if carry clear

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/19/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

if (0)	; moved into the resource that really uses it...jim 7/8/93
EC <serialDriver	char "serial  Edrv", 0			>
NEC <serialDriver	char "serial  drvr", 0			>
endif
;driverDir	char 0

PSerialSetDevice	proc	far
		uses	es, bx, dx, cx, ax, si, ds
		.enter
	; lock BIOS while we use it
		call	SysLockBIOS


		tst	ds:[baseAddress]
		jz	noCIS

	; if the card has a CIS then use the info we got from it to
	; configure the card
		call	PSerialSetConfiguration
		jc	err
noCIS:
	; now that the card is configured (if possible) set up the
	; PCMCIA port itself.

		mov	ah, IF_IO
		call	PSerialSetCard

err:
	; unlock BIOS as all well behaved drivers should
		call	SysUnlockBIOS
		jc	done		; carry set, something must be wrong

	; lets give the card some time to power up...
		mov	ax, 2		; 1/30 of a second
		call	TimerSleep

		mov	bx, di			;Save whether this is 
						; test or set device call
		mov	di, cx
		add	di, ds:[baseAddress]	; di = effective base address

	; If we are doing a SET_DEVICE call, then inform the serial driver/
	; power driver of our existence.

		cmp	bx, DRE_TEST_DEVICE
		je	done

ifidn HARDWARE_TYPE, <ZOOMER>
	; di = address of the port
	; now lets tell the power driver where we put the port so the
	; serial driver can get at the address later if it needs to

		call	PSerialNotifyPowerDriver
		jc	done		; no power driver, something is wrong

	; if the Serial Driver is already loaded we need to define ourselves
	; a port for the device
		call	PSerialFindSerialDriver
		jnc	done
		mov_tr	ax, di		; ax = base address
		mov	di, DR_SERIAL_DEFINE_PORT
		mov	cl, 8		; PCMCIA irq = 8 for Zoomer
		push	ax		; save address
		call	ds:[si].DIS_strategy
		pop	di		; return address in di
endif
		clc
done:
		.leave
		ret
PSerialSetDevice	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialSetConfiguration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	set up the card to its confiuration that we like

CALLED BY:	INTERNAL

PASS:		ds	= dgroup
		(Have BIOS lock)

RETURN:		carry on error

DESTROYED:	ax, bx, cx, dx

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/25/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialSetConfiguration	proc	near
		uses	es, di
		.enter

	; we know that the value in configIndex will select the configuraton
	; we want, we just write it out to configAddress
	; first we need to set the window to be a memory window

		mov	ah, IF_MEMORY
		call	PSerialSetCard
		jc	done

	; now set page zero to be at the beginning of attribute memory
		movdw	cxdx, ds:[configAddress]
		call	PSerialMapPage		; es:bx <- first register
		jc	done

		mov	al, ds:[configIndex]
		or	al, mask COR_levelRequest
		and	al, not mask COR_sreset
		mov	es:[bx], al
		test	ds:[registerMask], 02h	; check for register 1
		jz	done
		or	{byte}es:[bx+2], mask SRO_8BitBus
;check power - 8/12/93
		call	PSerialCheckCardPower	; returns carry
done:
		.leave
		ret
PSerialSetConfiguration	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialMapPage
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Map in a page of attribute memory for writing

CALLED BY:	(INTERNAL) PSerialSetConfiguration
PASS:		cxdx	= offset within attribute memory to map
RETURN:		carry set on error
		carry clear if ok:
			es:bx	= address of requested byte.
DESTROYED:	ax, cx, dx
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/19/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PSerialMapPage	proc	near
window		local	WindowData
winSeg		local	sptr
		uses	si, di
		.enter
		pushdw	cxdx
	;
	; Find the location of the memory window.
	; 
		clr	al, bh		; adapter 0, window 0
		segmov	es, ss
		lea	di, ss:[window]
		mov	ss:[window].WD_bufferLength, size window - offset WD_table
		mov	ah, SS_INQUIRE_WINDOW
		int	SOCKET_SERV_INTERRUPT
		jc	done
	;
	; Compute the segment address for easy access.
	; 
		mov	ax, ss:[window].WD_table.WDT_mem.MWT_firstByte
		mov	ah, al
		clr	al
		mov	ss:[winSeg], ax
		popdw	cxdx
	;
	; Save desired position for computing offset within window.
	; 
		mov	bx, dx		; just need low word as we couldn't
					;  cope with a difference > 64K anyway
	;
	; Round down to nearest 4K, at least, or to window size, if required
	; 
		mov	ax, 4096

;		test	ss:[window].WD_table.WDT_mem.MWT_capabilities,
;				mask MWC_BASE_ALIGN
;argh, let's assume that window size is fixed (i.e. MWC_SIZE is clear and size
;stored in MWT_maxSize, should check MWC_SIZE and use SS_GET_WINDOW if
;MWC_SIZE is set) - brianc 7/1/93
		tst	ss:[window].WD_table.WDT_mem.MWT_maxSize

		jz	computeWinBase
		mov	ax, ss:[window].WD_table.WDT_mem.MWT_maxSize
		ror	ax		; shift it left 12 bits to
		ror	ax		;  multiply by 4K
		ror	ax
		ror	ax
computeWinBase:
		dec	ax
		not	ax
		andnf	dx, ax		; cxdx <- base for mapping
		push	dx, bx
	;
	; Convert map base to 4K units (i.e. shrdw cxdx, 12)
	; 
		call	PCMConvertToPage
	;
	; Map the page in writable to attribute memory
	; 
		mov	dl, mask PS_ENABLED or mask PS_ATTRIBUTE
		clr	bx		; window 0, page 0
		mov	ax, SS_SET_PAGE shl 8
		int	SOCKET_SERV_INTERRUPT
		pop	dx, bx
		jc	done
	;
	; Figure the offset within the window from which to fetch the
	; byte.
	; 
		sub	bx, dx
		mov	es, ss:[winSeg]
		clc
done:
		.leave
		ret
PSerialMapPage	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialCheckCardPower
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Check power to card

CALLED BY:	(INTERNAL)
PASS:		nothing
RETURN:		carry set on error
		carry clear if ok
DESTROYED:	none
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	brianc	8/12/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PSerialCheckCardPower	proc	near
		uses	ax, bx, cx, dx, di
		.enter
	;
	; Use GET_SOCKET to check power
	; 
		clr	bl		; socket is zero
		mov	ax, SS_GET_SOCKET shl 8		; adapter is zero
		int	SOCKET_SERV_INTERRUPT
					; returns: ah, bh, ch, cl, dh, dl, di
		jc	done
		cmp	cx, 0		; power off?
		stc			; assume so
		je	done		; yes
		clc			; else, okay
done:
		.leave
		ret
PSerialCheckCardPower	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialSetCard
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	set up the card for I/O mode and set base address
		for I/O window useing socket services

CALLED BY:	GLOBAL

PASS:		ah = Interface Type (IF_MEMORY or IF_IO)

RETURN:		cx = base address

DESTROYED:	ax,bx, dx

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/25/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialSetCard	proc	near	uses	di
win	local	WindowData
sid	local	SocketInterfaceData
		.enter
		clr	al	
		mov	sid, ax

	; since I am calling this from TestDevice di will tell me
	; whether I am being called for TEST  or SET device
	; lets inquire about the window
		clr	al
		mov	bh, al
		lea	di, win
		segmov	es, ss
		mov	win.WD_bufferLength, size MemIOWindowTable
		mov	ah, SS_INQUIRE_WINDOW
		int	SOCKET_SERV_INTERRUPT

	; if the window doesn't support I/O access, we can't handle it.

		test	bx, mask GWC_IO
		stc
		jz	exit

	; switch the interface to the correct mode, leaving the interrupt
	; disabled for now
		push	bx			; save capabilities
		clr	al	; one one adapter
		mov	bl, al	; only one socket
		mov	ah, SS_GET_SOCKET
		int	SOCKET_SERV_INTERRUPT

		clr	di
		ornf	di, sid	;Set socket to IO or mem mode
		cmp	sid, IF_IO shl offset SID_TYPE
		jne	setMem
		ornf	di, mask AIRQ_HIGH or 8 	;(int level 8)
		
setMem:
		mov	ah, SS_SET_SOCKET
		clr	al	; one one adapter (trashed by GET_SOCKET)
		mov	bl, al	; only one socket
		int	SOCKET_SERV_INTERRUPT
		pop	bx
		jc	exit

	; ok, lets see if its is just an IO port, or it is both so we know
	; where to get the data returned by INQUIRE_WINDOW from

		test	bx, mask GWC_ATTRIBUTE or mask GWC_COMMON
		jnz	bothDevices

		lea	di, win.WD_table.WDT_io		
		cmp	sid, IF_IO shl offset SID_TYPE
		je	gotFirstByte
		lea	di, win.WD_table.WDT_mem
		jmp	gotFirstByte
bothDevices:
		lea	di, win.WD_table.WDT_memIo.MIOWT_io
		cmp	sid, IF_IO shl offset SID_TYPE
		je	gotFirstByte
		lea	di, win.WD_table.WDT_memIo.MIOWT_mem
gotFirstByte:
		mov	ax, ss:[di].IOWT_capabilities
		mov	bx, ss:[di].IOWT_firstByte
		mov	cx, ss:[di].IOWT_minSize

	; now let us configure the port window to be I/O at firstByte

		mov	di, bx		
		clr	ax
		mov	bx, ax	; bl = socket, bh = window (both zero)
		mov	ah, SS_SET_WINDOW
		mov	dh, mask WS_ENABLE
		cmp	sid, IF_IO shl offset SID_TYPE
		jnz	notIO
		mov	dh, mask WS_IO or mask WS_ENABLE ; state
notIO:
		mov	dl, DS_250NS	; speed = 250 nano-seconds (slowest)
		int	SOCKET_SERV_INTERRUPT

exit:
		.leave
		ret
PSerialSetCard	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialCISEnumCallback
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	call back to deal with CIS entries

CALLED BY:	PCMEnumCIS

PASS:		es:di 	= tuple data
  		cx	= # data bytes
		ax	= CISTuple

RETURN:		carry set to stop processing

DESTROYED:	es, cx, dx, ax, di, si

PSEUDOCODE/STRATEGY:	

		the idea here is to find a configuration of the card that
		we can support.  so just go along looking for confiuration
		informatoin in the form of a configuration tuple which will
		contain various configurations that the card supports. Take
		first one that looks like it will work for us and use it.

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/21/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PSerialCISEnumCallback	proc	far
		uses	ds, si
		.enter
		segmov	ds, es
		mov	si, di		; ds:si = data
		cmp	al, CISTPL_CONFIG
		jne	tryConfigTable
	; processing the config tuple will get the configAddress we need
	; to configure the card once we decide on a configuration
		call	PSerialProcessConfigTuple
		jmp	doneOK
tryConfigTable:
		cmp	al, CISTPL_CFTABLE_ENTRY
		jne	doneOK
		call	PSerialProcessConfigTableEntry
doneOK:
		clc
		.leave
		ret
PSerialCISEnumCallback	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialProcessConfigTableEntry
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	process a config table entry

CALLED BY:	PSerialCISEnumCallback

PASS:		es:di 	= tuple data
  		cx	= # data bytes
		ax	= CISTuple

RETURN:		carry set to stop processing
		the index value for the desired configuration 

DESTROYED:	cx, dx, ax, si

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/24/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialProcessConfigTableEntry	proc	near
features	local	byte
		uses	bx, cx, dx
		.enter
	; if the configIndex is non-zero than we must have already set
	; it up, so do nothing
		tst	es:[di].ED_configIndex
		jnz	done

	; ok, lets see if this is a configuration we can support
	; if so get its index value into configIndex
		lodsb		; al = index
		mov	es:[di].ED_configIndex, al
	; if there is no interface type then forget it as default is memory
	; and we need I/O
		test	al, mask CTI_interface
		jz	done
		lodsb		; al = InterfaceType
		and	al, mask CTIFD_IFType

	; we now have the Interface type, if its not I/O then forget it
		cmp	al, 1 ; IF_IO - 1
		jne	done
		
		lodsb		; al = feature selection byte
		mov	features, al

		test	al, mask FS_power
		jz	timing
	; ok, lets do a power descriptor
		mov	bl, al
		and	bl, mask FS_power
	; Vcc power
		call	PSerialProcessPowerDescriptor
		cmp	bl, 1
		je	timing
	; Vpp1 power
		call	PSerialProcessPowerDescriptor
		cmp	bl, 2
		je	timing
	; Vpp2 power
		call	PSerialProcessPowerDescriptor
timing:
		test	al, mask FS_timing
		jz	getIo
		call	PSerialGetTiming
	; the card must be able to work off of an 8 bit bus as thats all
	; Zoomer will allow
getIo:
		test	features, mask FS_ioSpace
		jz	getIrq
		call	PSerialGetIo
		jc	error
	; ALL THIS STUFF IS BEING IGNORE FOR NOW
	; the IRQ information is pretty useless for the ZOOMER and
	; this is an IO driver so the MemSpace is not so useful either
	; the misc stuff may someday be useful, who knows?
getIrq:
if 0
		test	features, mask FS_irq
		jz	getMemSpace
		call	PSerialGetIrq
getMemSpace:
		test	features, mask FS_memSpace
		jz	getMisc
		call	PSerialGetMemSpace
getMisc:
		test	features, mask FS_misc
		jz	doneOK
		call	PSerialGetMisc	; not yet written...
endif
;doneOK:
		clc
done:
		.leave
		ret
error:
		mov	es:[di].ED_configIndex, 0
		stc
		jmp	done
PSerialProcessConfigTableEntry	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialGetMemSpace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get mem space info for a config table entry

CALLED BY:	PSerialProcessConfigTableEntry

PASS:		ds:si = mem space info

RETURN:		ds:si = following info

DESTROYED:	ax

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:
		this is an IO driver so just read this info without
		processing it

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/27/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
ifidn HARDWARE_TYPE, <NOONE>
PSerialGetMemSpace	proc	near
		uses	bx, cx, dx, si, di, bp
		.enter
		lodsb
		tst	al
		jz	done		; 0 = no memory mapping required
		cmp	al, 1
		jne	try2
		lodsw			; read in length in 256 byte pages
		jmp	done		
try2:
		cmp	al, 2
		jne	do4
		lodsw			; read in length
		lodsw			; read in address
		jmp	done
do4:
		lodsb			; memory space descriptor byte
		mov	bp, ax
		mov	cl, al
		and	cl, mask MSD_windows
		clr	ch		; cx = number of window descriptors
		jcxz	done
		mov	si, cx
		mov	bl, al
		and	bl, mask MSD_lengthSize
		mov	cl, offset MSD_lengthSize
		shr	bl, cl		; bl = size of lengths in bytes
		mov	dl, al
		and	dl, mask MSD_cardAddressSize
		mov	cl, offset MSD_cardAddressSize
		shr	dl, cl		; dl = card address size in bytes
		clr	di
		mov	di, dx
		clr	ch
		mov	cl, al
		and	cl, mask MSD_hostAddress
		mov	bp, cx
windowLoop:
		tst	si
		jz	done
		dec	si
		mov	cl, bl
		jcxz	noLength
		dec	cx
		call	PSerialGetNBytesOfDword
noLength:
		mov	cx, di
		jcxz	noAddress
		dec	di
		call	PSerialGetNBytesOfDword	
noAddress:		
		tst	bp
		jz	windowLoop
		mov	cx, di
		jcxz	windowLoop
		dec	di
		call	PSerialGetNBytesOfDword	
		jmp	windowLoop
done:
		.leave
		ret
PSerialGetMemSpace	endp
endif


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialGetIo
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get io info from a config table entry

CALLED BY:	PSerialProcessConfigTableEntry

PASS:		ds:si = io info

RETURN:		ds:si = following data
		carry set on error

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/27/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialGetIo	proc	near
		.enter
		lodsb	; get io address linesinfo
		test	al, mask IOSD_bus8
		jz	error
		test	al, mask IOSD_range
		jnz	getRange
	; the thing can respond to all address in its address space so
	; just put it at 03f8h (COM1) since it can go anywhere
noAddressBytes:
		mov	es:[di].ED_baseAddress, 3f8h
		jmp	doneOK
getRange:
		lodsb	; get range info byte
		mov	cl, offset IORD_addressSize
		mov	dl, al
		and	dl, mask IORD_addressSize
		shr	dl, cl		; dl = address size
		mov	cl, dl
		jcxz	noAddressBytes
		cmp	cx, 3
		je	getAddress
		dec	cx
getAddress:
		call	PSerialGetNBytesOfDword
	; if dx is non-zero we are way out of range
		tst	dx
		jnz	error
		mov	es:[di].ED_baseAddress, ax
doneOK:
		clc
done:
		.leave
		ret
error:
		stc
		jmp	done
PSerialGetIo	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialGetIrq
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get irq info from a config table entry

CALLED BY:	PSerialProcessConfigTableEntry

PASS:		ds:si = irq info

RETURN:		ds:si = following data

DESTROYED:	ax

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/27/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
ifidn HARDWARE_TYPE, <NOONE>
PSerialGetIrq	proc	near
		.enter
	; since this info is not really useful to use as they are only
	; suggestions, we will just read it in
		lodsb
		test	al, mask ID_mask
		jz	done
		lodsw	; next two bytes are IRQ 0-15 mask
done:
		.leave
		ret
PSerialGetIrq	endp
endif

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialGetTiming
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get timing info from a config tablle entry

CALLED BY:	PSerialProcessConfigTableEntry

PASS:		ds:si = timing info

RETURN:		ds:si = following data

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/27/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialGetTiming	proc	near
		.enter
		lodsb	; get timing info
		mov	bl, al
		and	bl, mask TI_waitScale
		cmp	bl, 3 shl offset TI_waitScale
		je	noWaitScaleBytes
		call	PSerialGetExtNumber
noWaitScaleBytes:
		mov	bl, al
		and	bl, mask TI_readyBusyScale
		cmp	bl, 7 shl offset TI_readyBusyScale
		je	noRBScale
		call	PSerialGetExtNumber
noRBScale:
		mov	bl, al
		and	bl, mask TI_reservedScale
		cmp	bl, 7 shl offset TI_reservedScale
		je	done
		call	PSerialGetExtNumber
done:
		.leave
		ret
PSerialGetTiming	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialGetExtNumber
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get an exteneded value 

CALLED BY:	GLOBAL

PASS:		ds:si = value

RETURN:		ds:si = byte after extened value

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/24/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialGetExtNumber	proc	far
		uses	ax
		.enter
extLoop:
		lodsb
		test	al, 80h
		jnz	extLoop
		.leave
		ret
PSerialGetExtNumber	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialProcessPowerDescriptor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	deal with a power descriptor

CALLED BY:	PSerialProcessConfigTableEntry

PASS:		ds:si = power descriptor

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	
			for now just advance past it...
KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/24/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialProcessPowerDescriptor	proc	near
		uses	bx, ax
		.enter
	; first grab the power description selection byte
		lodsb
		mov	bl, al
		mov	cx, 8
powerLoop:
		test	bl, 1
		jz	next
extLoop:
		lodsb
		test	al, mask PDV_ext
		jnz	extLoop
next:
		shr	bl
		loop	powerLoop
		.leave
		ret
PSerialProcessPowerDescriptor	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialProcessConfigTuple
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	deal with a configuration tuple

CALLED BY:	PSerialCISEnumCallback

PASS:		es:di 	= tuple data
  		cx	= # data bytes

RETURN:		carry set to stop processing
		the address of configure register 0 in configAddress

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/24/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialProcessConfigTuple	proc	near
		uses	ds, si, di, ax, bx, cx, dx
		.enter
	; first thing in the control tuple is the link to the next tuple
		lodsb
		mov_tr	bx, ax		; save ConfisFieldSizes record in bl
		lodsb	;	al = index of last entry in card config table
		mov	cx, bx

	; the address is 1-4 bytes long as specific in the ConfigFieldSizes
	; record, so get the correct number of bytes and put the address into
	; dx:ax leaving ds:si pointing to the next field in the tuple

		and	cx, mask CFS_ADDR_SIZE ; cx = size of register address
		call	PSerialGetNBytesOfDword
		movdw	es:[di].ED_configAddress, dxax
		and	bx, mask CFS_MASK_SIZE
		mov	cl, offset CFS_MASK_SIZE
		shr	bx, cl	; bx = # of bytes in register mask
		clr	ah

	; for now I only know the use of registers 0 & 1 so ignore the rest...
		lodsb	; there will always be >= 1 register present bytes

	; if the low bit of the first byte here is zero then this is a
	; non-configurable card, so forget it
		test	al, 1
		jz	error
		tst	bx
		jz	gotRegs
		mov	ah, ds:[si]
gotRegs:
		mov	es:[di].ED_registerMask, ax
		clc
done:		
		.leave
		ret
error:
		stc
		jmp	done
PSerialProcessConfigTuple	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialGetNBytesOfDword
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get from 1 to four bytes of a dword

CALLED BY:	GLOBAL

PASS:		ds:si = buffer holding 1 to 4 bytes
		cx = # of bytes to get - 1

RETURN:		dx:ax = dword zero padded to fill in non-existant bytes
		ds:si = ds:si+cx+1 (using inputed cx and si)

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/24/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialGetNBytesOfDword	proc	near
		uses	cx
		.enter
		clr	ax
		mov	dx, ax		; dx:ax = 0:0, will end up as address
		lodsb	; al = first byte of register address
		jcxz	gotAddress
		mov	ah, ds:[si]
		inc	si
		dec	cx
		jcxz	gotAddress
		mov	dl, ds:[si]
		inc	si
		dec	cx
		jcxz	gotAddress
		mov	dh, ds:[si]
		inc	si
gotAddress:
		.leave
		ret
PSerialGetNBytesOfDword	endp

Init	ends


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialConfirmRemoval
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	deal with the card being pulled out

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		carry set if card may not be removed yet
			^lbx:si	= string to display in error
				  notification box
		carry clear if ok
			^lbx:si	= string to display if previous
				  removal confirmation returned an
				  error. If bx is 0, the caller
				  should pick a suitable message.


DESTROYS:	di

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/20/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialConfirmRemoval	proc	far
		uses	cx
		.enter
		push	ds
		call	PSerialFindSerialDriver
	; if the is no serial driver then we don't really care if they take
	; the card out...
		jnc	popDoneOK
ifidn HARDWARE_TYPE, <ZOOMER>
	; in the Zoomer case pcmcia serial port is always COM4 so
	; lets see if COM4 is open, if so report that the port is in use
		mov	di, DR_SERIAL_STAT_PORT
		mov	bx, SERIAL_COM4		
		call	ds:[si].DIS_strategy

	; on carry set, the port doesn't even exist so there is nothing
	; for us to do
		jc	popDoneOK

	; we know the port exists, so lets see if its in use
	; if so, report that to the user and leave it open
		tst	ah	
		jnz	error

	; ok, the port is not in use, and the card has been taken out so
	; lets un-define the port for the serial driver's sake
		mov	cl, SERIAL_PORT_DOES_NOT_EXIST
		mov	di, DR_SERIAL_DEFINE_PORT
		call	ds:[si].DIS_strategy
		pop	ds
		push	ds

	; now we can tell the power driver that the port it gone
		mov	di, SERIAL_PORT_DOES_NOT_EXIST
		call	PSerialNotifyPowerDriver

		mov	dx, ds:[baseAddress]
		add	dx, 0800h	;Yeah, Jimmy, this is really great.
					; What the hell is 800h?? -atw
					;This clears the carry
		call	PSerialGetState
endif
popDoneOK:
		pop	ds
		mov	bx, 0		; preserve carry 
		.leave
		ret
ifidn HARDWARE_TYPE, <ZOOMER>
error:
		mov	bx, handle StringsUI
		mov	si, offset removalErrorString
		stc
		jmp	popDoneOK
endif
PSerialConfirmRemoval	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialGetState
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	

CALLED BY:	GLOBAL

PASS:		dx = base address for port
		ds = dgroup

RETURN:		baud rate divisor saved in baudRate
		line control info in lineControl

DESTROYED:	ax, dx: preserves flags

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	6/11/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PSerialGetState	proc	near
		.enter
		pushf
	; first set the DLAB so we can get the baud rate divisor
		add	dx, offset SP_lineCtrl		; offset line control
		in	al, dx
		ornf	al, mask SF_DLAB
		out	dx, al

	; now get the baud rate divisor
		add	dx, offset SP_divLow - offset SP_lineCtrl
		in	ax, dx
		mov	ds:[baudRate], ax
		add	dx, offset SP_lineCtrl - offset SP_divLow
	;
	; Fetch the transmission format and turn off access to the
	; baud rate divisor at the same time.
	;
 		in	al, dx
		andnf	al, not mask SF_DLAB
		out	dx, al

		mov	ds:[lineControl], al
		popf
		.leave
		ret
PSerialGetState	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialSetState
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	restore the state of the port

CALLED BY:	GLOBAL

PASS:		dx = base address of port
		ds = dgroup

RETURN:		Void.

DESTROYED:	ax, dx: preserves flags

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	6/11/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialSetState	proc	near
		.enter
		pushf

	; first set the DLAB so we can get the baud rate divisor
		add	dx, offset SP_lineCtrl	; offset line control
		in	al, dx
		ornf	al, mask SF_DLAB
		out	dx, al

	; now set the baud rate divisor
		add	dx, offset SP_divLow - offset SP_lineCtrl	
		mov	ax, ds:[baudRate]
		out	dx, ax
		add	dx, offset SP_lineCtrl - offset SP_divLow
	;
	; set the transmission format and turn off access to the
	; baud rate divisor at the same time.
	;
		mov	al, ds:[lineControl]
		out	dx, al

		popf
		.leave
		ret
PSerialSetState	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialCardReInserted
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	deal with the card reinserted call from the power driver

CALLED BY:	DR_PCMCIA_CARD_REINSERTED

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/20/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialCardReInserted	proc	far
		.enter
		push	ds
		call	PSerialFindSerialDriver
		mov	di, DR_SERIAL_DEFINE_PORT
		mov	cl, SERIAL_PORT_DOES_NOT_EXIST	; undefine the port
		call	ds:[si].DIS_strategy
		pop	ds
	; the DEFINE_PORT will return carry set if the port is
	; still in use which is what we want to return


	; now reconfigure the card which might have lost power from being
	; taken out...
		mov	di, DRE_SET_DEVICE
		call	PSerialSetDevice

	; restore the port now that the card has beeen reconfigured
		call	PSerialSetState
		.leave
		ret
PSerialCardReInserted	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialNotifyPowerDriver
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	tell the power driver the card now eixsts/does not exists

CALLED BY:	PSerialSetDevice, PSerialConfirmRemoval

PASS:		di = new base address for port

RETURN:		carry set if no power driver

DESTROYED:	ax, bx

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/24/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialNotifyPowerDriver	proc	far
		uses	di, ds, si
		.enter
		mov	ax, GDDT_POWER_MANAGEMENT
		call	GeodeGetDefaultDriver
		tst	ax
		jz	error
		mov_tr	bx, ax			; bx = driver
		call	GeodeInfoDriver
		mov	bx, di		; put base address in bx
		mov	ax, 1		; set new base address
		mov	cx, handle 0	;Handle of this driver
		mov	di, DR_POWER_SERIAL_PCMCIA_HACK
		call	ds:[si].DIS_strategy
		clc	
done:
		.leave
		ret
error:
		stc
		jmp	done
PSerialNotifyPowerDriver	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialFindSerialDriver
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	find the serial driver if its loaded

CALLED BY:	PSerialTestDevice, PSerialConfirmRemoval

PASS:		nothing

RETURN:		if carry set then ds:si = stategy function
		else carry clear

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/20/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

EC <serialDriver	char "serial  Edrv", 0			>
NEC <serialDriver	char "serial  drvr", 0			>
;driverDir	char 0

PSerialFindSerialDriver	proc	far
		uses	es, di, ax, cx, dx, bx
		.enter
		segmov	es, cs
		mov	di, offset serialDriver
		mov	ax, 12
		clr	cx
		mov	dx, cx
		call	GeodeFind		; carry set if  found
		jnc	done
	; bx = the handle of the serial driver now
		call	GeodeInfoDriver	; ds:si = strategy routine
		stc
done:
		.leave
		ret
PSerialFindSerialDriver	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		PSerialDoNothing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	null function for function table

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description

	----	----		-----------
	jimmy	5/17/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

PSerialDoNothing	proc	far
	.enter
	clc
	.leave
	ret
PSerialDoNothing	endp

Resident ends



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		Strings resource
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

DeviceStrings		segment	lmem LMEM_TYPE_GENERAL
DriverExtendedInfoTable	<
		{},			; lmem header added by Esp
		length fsNameTable,	; Number of supported "devices"
		offset fsNameTable,	; Names of supported "devices"
		offset fsInfoTable	; FSDFlags
>

fsNameTable	lptr.char	pserialString
		lptr.char 	0	; terminate table

pserialString	chunk.char	"PCMCIA Serial Driver", 0

fsInfoTable	word	0

DeviceStrings	ends
