COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PROJECT:	PC GEOS
MODULE:		Com43 -- file transfer routines
FILE:		file.asm

AUTHOR:		Adam de Boor

ROUTINES:
	Name		Description
	----		-----------
	FileStartXfer	Begin the file transfer protocol

	
REVISION HISTORY:
	Name	Date	Description
	----	----	-----------
	ardeb	6/15/88	Initial Revision

DESCRIPTION:
	The routines in this file handle the VSFTP (Very Simple File Transfer
	Protocol) established for transfering files from a PC to a UNIX system
	over dependable serial lines.

	All I/O is routed through the Com module, understandably.

	$Id: file.asm,v 2.9 94/01/05 17:37:36 cassie Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FILEMODULE	= 1
		include	com43.def
program		segment


;
; I/O buffer. Data stored here before being written to or after being read from
; disk.
;
block		db	BUFFER_SIZE dup(?)	; I/O block buffer
		bugsym	block

;
; Place to stuff file name that follows command
;
filename	db	FILENAME_SIZE dup(?)
		bugsym	filename
fSize		dd	?
		bugsym	fSize
feedback	db	"/-\\|"

readCmd		db


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileStartXfer
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle a file transfer command

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Everything

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/15/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	FileStartXfer
FileStartXfer	proc	near
		push	cs
		pop	es	; Make sure es is pointing to our segment
		mov	err, 0	; clear the error flag
FSXCmdLoop:	call	ComRead
		jz	FSXCmdLoop
		mov	cx, ax

		mov	bx, 0
FSXNameLoop:
		call	ComRead
		jz	FSXNameLoop
		and	al, 7fh
		jz	FSX1
		mov	filename[bx], al
		inc	bx
		jmp	FSXNameLoop
FSX1:
		mov	filename[bx], al

		cmp	cl, RECV_COMMAND
		jne	FSX2
read:
		mov	readCmd, cl
		call	FileReceive
		jmp	short FSXReturn
FSX2:
		cmp	cl, FETCH_COMMAND
		jne	FSX3
		mov	al, FILE_SEND_NORMAL
		call	FileSend
		jmp	short FSXReturn
FSX3:
		cmp	cl, RECV_VARIABLE
		je	read
		cmp	cl, FETCH_RAW_COMMAND
		jne	FSXReturn
		mov	al, FILE_SEND_RAW
		call	FileSend
		;
		; What to do. What to do...
		;
FSXReturn:
		cmp	err, 0
		jz	okMessage
		call	ScrWriteAbortString
		jmp	userMessage
okMessage:
		call	ScrWriteXferComplete
userMessage:
		mov	al, pc_unix
		cmp	al, PCCOM
		jne	done
		call	ScrWriteUserString
done:
		ret
FileStartXfer	endp




COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileReceiveStart
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle start of file receive, whether for variable or
		known-length file...

CALLED BY:	FileReceive, FileReceiveVariable
PASS:		filename	name read from other side
RETURN:		carry set if destination file couldn't be created
		bx	= file handle of destination
DESTROYED:	ax, cx, bx, dx

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	6/ 3/90		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
FileReceiveStart proc	near
		.enter
	;
	; First set the port into RAW mode without flushing anything
	;
		mov	cx, 1
		clr	ax		; Don't flush anything
		call	ComSetRawMode

	;
	; Tell the user what file we're receiving
	;
		mov	bx, offset receiving
		call	ScrWriteString
		mov	bx, offset filename
		call	ScrWriteString
		mov	al, ' '
		call	ScrWrite

	;
	; Create the file.
	;
		mov	ah, DOS_FILE_CREATE		; CREATE
		clr	cx		; normal file
		mov	dx, offset filename
		int	21h
		jc	error

		mov	bx, ax

		clc
done:
		.leave
		ret
error:
	;
	; Print the error code and an error message, then send a NAK back
	; to the host (to signal the failure) before returning carry set.
	; 
		push	bx
		mov	bx, offset createErr
		call	ScrPrintByte
		call	ScrWriteString
		pop	bx
		mov	al, NAK_QUIT
		call	ComWrite
		mov	err, 1
		stc
		jmp	done
createErr	db	": error creating file", 0
FileReceiveStart endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileCompareChecksum
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	See if the checksum that was sent by the host matches the
		one we've calculated in 'checksum'
		
		VESTIGIAL. RECEIVED CHECKSUM IS IGNORED.

CALLED BY:	FileReceive
PASS:		nothing
RETURN:		JNE if checksums mismatch
DESTROYED:	dx, di, cx

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	6/ 3/90		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
FileCompareChecksum	proc	near
		.enter
	;
	; Just read the thing. the checksum is vestigial. -- ardeb 4/28/93
	; 
		mov	cx, 2
		sub	sp, cx
		mov	di, sp
		call	FileReadNBytes
		pop	dx
		clr	dx
		.leave
		ret
FileCompareChecksum	endp

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileReceive
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Receive a file from The Other Side

CALLED BY:	FileStartXfer

PASS:		filename	name read from other side

RETURN:		Nothing

DESTROYED:	Lots of things.

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/16/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
receiving	db	cr, lf, "Receiving: ", 0
tries		db	0

FileReceive	proc	near

	;
	; Acknowledge transfer start with <0xff> character. Makes
	; sure we're all in sync.
	;
		mov	al, SYNC
		call	ComWrite
	;
	; Don't create the file yet.  Wait to see if the first block
	; contains the filename.  This flag will be cleared after the
	; first block is read, by FileReadBlock.
	;
		clr	bp		; start with first feedback char
		clr	bx 		; intialize file handle to 0
		ornf	sysFlags, mask SF_FIRST_BLOCK
	;
	; Now read the file size and initialize loop variables.
	;
varReadLoop:
		mov	di, offset fSize
		mov	cx, size fSize
		call	FileReadNBytes
		test	sysFlags, MASK SF_EXIT or MASK SF_NUKE_FILE
		jnz	cancel
fixedReadLoop:
		mov	ax, fSize.low
		or	ax, fSize.high
		jz	readComplete
    if	DEBUG
		mov	ax, fSize.low
		call	DebugPrintByte
    endif
	;
	; Figure the number of bytes to read this time.
	; This is the size of our buffer, unless there aren't
	; that many bytes left in the file, in which case we read
	; that many.
	;
		mov	cs:[tries], 0
readBlock:
		mov	di, offset block
		call	FileReadBlock
		jnc	gotBlock

		test	ds:[sysFlags], mask SF_EXIT 
	LONG	jnz	cancel

	; if we didn't get the block for some reason, we should send out
	; a request to resend the whole block and try it again
	; AFTER 3 TRIES WE WILL GIVE UP
readFailed:
		cmp	al, NAK_QUIT		
		je	errAndDone
		cmp	cs:[tries], MAX_TRIES
		jl	tryAgain
errAndDone:
		mov	al, NAK_QUIT
		mov	err, 1
		jmp	closeDone
tryAgain:
		inc	cs:[tries]
	;
	; Flush the input buffer so there is no junk in it if
	; the sender tries to send it again.  (Setting raw mode
	; won't hurt anything - it only sets a flag which is
	; already set.)
	;
		mov	cx, 1			; (re)set raw mode
		mov	ax, 1			; flush input buffer only
		call	ComSetRawMode		; 
		mov	al, NAK
		call	ComWrite
    if	DEBUG
		mov	ah, SCREEN_ATTR_INV
		mov	al, 'R'
		call	DebugPrintChar
		mov	al, cs:[tries]
		call	DebugPrintByte
    endif

		jmp	readBlock
gotBlock:
		test	sysFlags, MASK SF_EXIT or MASK SF_NUKE_FILE
		jnz	cancel

		tst	bx			; was file created?
		jz	createFile

continue:
	;
	; Put up the next feedback character for the user to see.
	;
		push	bx
		mov	al, feedback[bp]
		call	ScrWrite
		mov	al, bs
		call	ScrWrite
		inc	bp
		and	bp, 3
		pop	bx
		mov	cs:[tries], 0
	;
	; Write the bytes out. BX already contains file handle.
	; CX contains number of bytes read (b/c of POP).
	;
		mov	ah, DOS_FILE_WRITE		; WRITE
		mov	dx, offset block
		int	21h
		jc	FRWError
		cmp	ax, cx
		jne	FRWError

		;
		; Subtract the number of bytes written from the size of
		; the file. If the result is non-zero, there's more to read.
		;
		sub	fSize.low, cx
		jnc	FR4
		dec	fSize.high
FR4:
		mov	al, SYNC
		cmp	readCmd, RECV_COMMAND
		je	ackBlock

		;
		; Compare checksums
		;
		call	FileCompareChecksum
		jne	FRCSError
		
		mov	al, ACK
ackBlock:
		call	ComWrite
		
	;
	; Jump back to the proper place in the loop, based on whether we
	; know the absolute size of the file, as evidenced by the command
	; we stored in FileStartXfer
	;
		cmp	readCmd, RECV_COMMAND
		je	fixedReadLoop
		jmp	varReadLoop

readComplete:
	;
	; File has been completely read. If command was RECV_COMMAND, we've
	; an overall checksum to verify before we can acknowledge the file.
	;
		cmp	readCmd, RECV_COMMAND
		jne	ackFile
		
		call	FileCompareChecksum
		jne	FRCSError
ackFile:
		mov	al, ACK
FRReturn:
		call	ComWrite	; Send response
closeDone:
		tst	bx
		jz	done
		mov	ah, DOS_FILE_CLOSE	; CLOSE
		int	21h
done:
		call	ComDrainQueue
		clr	cx
		clr	ax
		call	ComSetRawMode
		;
		; Go to next line...
		;
		mov	bx, offset crlf
		call	ScrWriteString
		ret

createFile:
		call	FileReceiveStart	; bx <- file handle
		jc	cancel
	;
	; If the first block did *not* contain the filename, it is a
	; normal data block and we should treat it as such - jump back
	; into the proper place in the loop.
	;
		andnf	ds:[sysFlags], not (mask SF_FIRST_BLOCK)
		test	ds:[sysFlags], mask SF_READ_FILENAME
	LONG	jz	continue

	;
	; If the first block did contain the filename, we need to acknowledge
	; its receipt here, since we will be going to the start of the
	; loop and it would not be acknowledged.
	;
		mov	al, SYNC		; send a SYNC
		call	ComWrite
		andnf	ds:[sysFlags], not (mask SF_READ_FILENAME)
	;
	; Jump back to the proper loop, based on whether we know the
	; absolute size of the file, as evidenced by the command
	; (net-based xfer uses RECV_VARIABLE)
	;
		cmp	ds:[readCmd], RECV_COMMAND	
	LONG	je 	fixedReadLoop
		jmp	varReadLoop

FRError:
		call	ScrPrintByte
		call	ScrWriteString
		pop	bx
cancel:
		mov	err, 1
		mov	al, NAK_QUIT
		jmp	FRReturn
FRWError:
		push	bx
		mov	bx, offset writeErr
		jmp	FRError
writeErr	db	": error writing file", 0
FRCSError:
		push	bx
		mov	bx, offset csErr
		jmp	FRError
csErr		db	": checksum error", 0
crlf		db	cr, lf, 0
FileReceive	endp
		bugsym	FileReceive


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileReadNBytes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get N bytes, ignore special chars

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 5/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

FileReadNBytes	proc	near
	.enter
		cld
		jcxz	done
if	DEBUG
		mov	al, 'N'
		mov	ah,SCREEN_ATTR_INV
		call	DebugPrintChar
endif
FileReadBlock1:
		call	ComReadWithWait
		jc	timeOut
		stosb
		test	sysFlags, MASK SF_EXIT or mask SF_NUKE_FILE
		loope	FileReadBlock1
		clc
done:
	.leave
	ret
timeOut:
		stc
		jmp	done
FileReadNBytes	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileSendBinary
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Send a file as 8-bit binary data.

CALLED BY:	FileSend
PASS:		BX	= handle of file
RETURN:		Nothing
DESTROYED:	AX, CX, DX, block

PSEUDO CODE/STRATEGY:
	Read the file in 1K blocks and write them until we hit EOF, summing
	all the bytes into a single-word checksum. After each block we
	synchronize with the host.


KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	6/24/88		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	FileSendBinary
FileSendBinary	proc	near
tries	local	word
		.enter
		clr	di
FSB1:
		;
		; Provide feedback
		;
		mov	tries, 0
		mov	al, feedback[di]
		call	ScrWrite
		mov	al, bs
		call	ScrWrite
		inc	di
		and	di, 3

		;
		; Read the next block
		;
		mov	dx, offset block
		mov	cx, size block
		mov	ah, DOS_FILE_READ	; READ
		int	21h
		jc	FSBRet		; C set on error

		mov	cx, ax		; Get number of bytes read in cx
		jcxz	FSBDone		; 0 => EOF
sendBlock:
		mov	si, offset block
		push	bx		; save file handle
		call	WriteDataBlock	; Write the whole block at once
		pop	bx		; restore file handle
	;
	; Wait for the other side to say the packet got there.
	; 
		call	Sync
		jc	tryAgain		; timeout
		cmp	al, NAK
		je	tryAgain		; rejected
		cmp	al, NAK_QUIT
		je	error			; give up
		jmp	FSB1			; do next block

tryAgain:
	;
	; only try so many times, then give up
	;
		inc	tries
		cmp	tries, MAX_TRIES
		je	error
		jmp	sendBlock
FSBDone:
	;
	; Always write a 0 checksum as the other side doesn't care about it.
	; 
		clr	al
		call	ComWrite
		call	ComWrite

FSB3:
		test	sysFlags, MASK SF_EXIT or MASK SF_NUKE_FILE
		jnz	FSBRet
		call	ComRead
		jz	FSB3
		tst	al
		jz	FSBRet
		push	ax
		mov	bx, offset transmitError
		call	ScrWriteString
		pop	ax
		call	ScrPrintByte
		stc
FSBRet:
		.leave
		ret
error:
		mov	err, 1
		stc
		jmp	FSBRet
transmitError	db	"Error sending file: ", 0
FileSendBinary	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileSendText
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Send a file as text, mapping ^M^J sequences into just ^J

CALLED BY:	FileStartXfer
PASS:		BX	= handle of file
RETURN:		Nothing
DESTROYED:	AX, CX, DX, block

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	6/24/88		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	FileSendText
count		dw	0		; Number of characters remaining in
					; current output block
FileSendText	proc	near
crc	local	word
file	local	word	
tries	local	word
blockPos	local	dword		; position of first byte in current
					;  packet within the file
		.enter
		mov	file, bx
		clr	di
		clr	cx		; none left from previous block

startBlock:	; send another block
	;------------------------------------------------------------
	;
	;		PACKET LOOP
	;
		mov	crc, 0
		mov	tries, 0
		
	;
	; Record the current position, in case we have to retransmit the
	; block.
	; 
		push	cx
		clr	cx
		clr	dx
		mov	ax, (DOS_FILE_SEEK shl 8) or SEEK_CURRENT
		mov	bx, file
		int	21h
		mov	ss:[blockPos].low, ax
		mov	ss:[blockPos].high, dx
		pop	cx

	;
	; first send off the start character
	;
		mov	al, BLOCK_START
		call	ComWrite

		;
		; Start sending a block of characters. We provide feedback
		; for every block sent
		;
		mov	count, BUFFER_SIZE	; Initialize output count
		mov	al, feedback[di]
		call	ScrWrite
		mov	al, bs
		call	ScrWrite
		inc	di
		and	di, 3
		tst	cx
		jnz	short FSTProcess; Characters left from last block --
					; go process them.
	;------------------------------------------------------------
	;
	;		BUFFER LOOP
	;
FSTNextBuffer:	; send next chunk of data (not neccessarily a "block")
		;
		; Get the next buffer of data from the the file
		;

		mov	dx, offset block
		mov	si, dx
		mov	cx, size block
		mov	ah, DOS_FILE_READ
		mov	bx, file
		int	21h
		jc	FSTRet
    if	DEBUG
		call	DebugPrintWord
    endif

		mov	cx, ax		; Want count in CX for looping
		jcxz	FSTDone		; 0 count => EOF
		;
		; Go through the block we just got ignoring carriage returns
		; and summing everything else into the checksum in dx. We
		; stop looping when either we run out of characters or we've
		; completed the current output block (count goes to 0)
		;
FSTProcess:
		clr	ah		; 0-extend the characters
FSTProcLoop:
		lodsb

		cmp	al, cr
		jne	processChar
		loop	FSTProcLoop
		jmp	FSTNextBuffer

processChar:
		mov_tr	bx, ax		; bl <- char
		mov	ax, crc
		call	IncCRC		; update CRC
		mov	crc, ax
		mov_tr	ax, bx		; al <- char again
	;
	; Take care of quoting characters that need quoting.
	; 
		cmp	al, BLOCK_START
		je	specialChar
		cmp	al, BLOCK_END
		je	specialChar
		cmp	al, BLOCK_QUOTE
		jne	sendChar

specialChar:
	;
	; we have a special char, so we must quote it. First comes the
	; quote character, then the quoted form of this character.
	;
		push	ax
		mov	al, BLOCK_QUOTE
		call	ComWrite
		pop	ax
		.assert (BLOCK_END eq BLOCK_START+1) and \
			   (BLOCK_END_DATA eq BLOCK_START_DATA+1)
		.assert (BLOCK_QUOTE eq BLOCK_END+1) and \
			   (BLOCK_QUOTE_DATA eq BLOCK_END_DATA+1)

		add	al, BLOCK_START_DATA - BLOCK_START
sendChar:
		call	ComWrite

		dec	count		; another byte used in the *packet*
		loopnz	FSTProcLoop
		jnz	FSTNextBuffer	; Ran out of characters -- fetch
					; next block.
	;
	; Output block complete -- synchronize with host and jump
	; to the top of our loop to reset count and give the user
	; feedback.
	;
    if	DEBUG
		mov	al, 'D'
		mov	ah,SCREEN_ATTR_INV
		call	DebugPrintChar
    endif
		mov	al, BLOCK_END
		call	ComWrite
	;
	; send out the CRC for the block a byte at a time
	;
		mov	ax, crc
		call	ComWrite
		mov	al, ah
		call	ComWrite
	;
	; Wait for acknowledgement of the packet.
	; 
		call	Sync
		jc	tryAgain		; timeout
		cmp	al, NAK
		je	tryAgain		; rejected
		cmp	al, NAK_QUIT
		je	error			; cancel transfer
		jmp	startBlock			; do the next block

tryAgain:
		inc	tries
		cmp	tries, MAX_TRIES
		je	FSTRet
	;
	; Seek back to the start.
	; 
		mov	cx, ss:[blockPos].high
		mov	dx, ss:[blockPos].low
		mov	ax, (DOS_FILE_SEEK shl 8) or SEEK_START
		mov	bx, file
		int	21h
	;
	; And go generate the packet again.
	; 
		jmp	startBlock

FSTDone:
		cmp	count, BUFFER_SIZE
		je	FST3
		;
		; Got out because we hit EOF but output block wasn't complete,
		; so complete it and wait for a sync byte from the other side.
		;
    if	DEBUG
		mov	al, 'd'
		mov	ah,SCREEN_ATTR_INV
		call	DebugPrintChar
    endif
		mov	al, BLOCK_END
		call	ComWrite

	; send out the CRC for the block
		mov	ax, crc
		call	ComWrite
		mov	al, ah
		call	ComWrite

		call	Sync
		jc	error
		cmp	al, SYNC
		jne	error

FST3:
	ifdef FILE_DEBUG
		mov	al, '!'
		call	ScrWrite
	endif

	;
	; Put out 0 checksum, always... why not get rid of the thing entirely?
	; I dunno...
	; 
		clr	al
		call	ComWrite
		call	ComWrite
FST2:
		;
		; Wait for the acknowledgement from the host. We allow the
		; user to quit out of it using either Sys-f or Sys-Q.
		;
		test	sysFlags, MASK SF_EXIT or MASK SF_NUKE_FILE
		jnz	FSTRet
		call	ComRead
		jz	FST2
		tst	al
		jz	FSTRet
		push	ax
		mov	bx, offset transmitError
		call	ScrWriteString
		mov	err, 1
		pop	ax
		call	ScrPrintByte
		stc
FSTRet:
		.leave
		ret
error:
		mov	err, 1
		jmp	FSTRet
FileSendText	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileType
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Figure out what type of file is referenced by dta

CALLED BY:	FileSend
PASS:		BX	= Handle open to file described by dta
RETURN:		AX	= transfer function
		fSize	= number of bytes to transfer
		CF	= 1 if error.
DESTROYED:	block, DX, CX

PSEUDO CODE/STRATEGY:
       Read through the file 1K at a time, counting the number of carriage
       returns. Each byte is or'ed into DL so at the end, bit 7 of DL will
       be set if the file is binary.
       
       If the file is text, we subtract the number of carriage returns from
       the size of the file (which we moved into fSize at the beginning). Note
       we assume that for each carriage return there's a matching linefeed...
       
       Once we've gone all the way through the file, we rewind the beast and
       use DL to figure the file type, setting AX and decrementing fSize to
       account for the carriage returns we'll be nuking (if it's text).
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	6/24/88		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	FileType
returns		dw	0		; Count of carriage returns in file
FileType	proc	near
		mov	ax, word ptr dta + dtaFileSize
		mov	word ptr fSize, ax
		mov	ax, word ptr dta + dtaFileSize + 2
		mov	word ptr fSize+2, ax
		
		mov	returns, 0
		clr	dl
FT1:
		push	dx		; Preserve Binary flag
		mov	dx, offset block
		mov	si, dx		; Load SI at the same time
		mov	cx, size block
		mov	ah, DOS_FILE_READ		; READ
		int	21h
		jc	FTRet
		
		pop	dx
		mov	cx, ax
		jcxz	FTDone		; EOF -- done with read.
FT2:
		lodsb
		or	dl, al		; Merge byte in. When all is done,
					; bit 7 of dl will be set if the
					; file is binary.
		cmp	al, cr		; Carriage return?
		jne	FT3
		inc	returns
FT3:
		loop	FT2
		jmp	FT1

FTDone:
		;
		; Figure out transfer function and reposition the file
		; at its beginning.
		; 
		push	dx

		clr	cx
		mov	dx, cx
		clr	al
		mov	ah, DOS_FILE_SEEK		; LSEEK
		int	21h
		
		pop	dx
		test	dl, 80h
		jnz	FTBin
		mov	ax, returns
		sub	word ptr fSize, ax
		jnc	FT4
		dec	word ptr fSize + 2
FT4: 		mov	ax, offset FileSendText
		jmp	short FT5
FTBin:
		mov	ax, offset FileSendBinary
FT5:
		clc
FTRet:
		ret
FileType	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		FileSend
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Send one or more files to the remote side.

CALLED BY:	FileStartXfer

PASS:		filename	name/pattern to send
		al = 0 : normal
		al = 1 : raw mode
RETURN:		Nothing

DESTROYED:	Everything

PSEUDO CODE/STRATEGY:
	The protocol the remote side expects is as follows:
		After filename is sent, wait for sync byte (0ffh)
		loop:
			get next byte
			if byte not RECV_COMMAND, transfer complete
			receive filename
			send sync byte
			receive file size
			receive data in 1K blocks with CRC sending a sync or
				NAK or NAK_QUIT byte after each
			receive checksum (ignored)
			send ACK if ok, NAK if not. NAK aborts transfer.
		:pool
			

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/16/88		Initial Revision
	jimmy	8/92		added raw mode

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
noSuchFile	db	": no such file", cr, lf, 0
cantOpen	db	"Can't open file", cr, lf, 0
sending		db	cr, lf, "Sending: ", 0
copyPosition	dw	0
rawMode		db	0
FileSend	proc	near
		mov	rawMode, al
		;
		; Acknowledge transfer start with <0xff> character. Makes
		; sure we're all in sync.
		;
		mov	al, SYNC
		call	ComWrite

		;
		; Set the transfer address where we know about it and have
		; a nice little structure set up for easy reference.
		; 
		mov	dx, offset dta
		mov	ah, 1ah		; Set disk transfer address
		int	21h
		
		;
		; Find the last backslash in the passed filename so we can
		; copy the tail into the buffer in the right place.
		;
		mov	di, offset filename
		mov	al, '\\'
		mov	cx, size filename
		cld
		mov	copyPosition, offset filename
FSBSLoop:	repne	scasb		; Search for next one
		jne	FS0_5		; No more => done
		mov	copyPosition, di; Save position after it...
		jmp	FSBSLoop
FS0_5:
		cmp	copyPosition, offset filename
		jne	FS0_7
		;
		; No backslash -- see if there's a colon after which we
		; should place the beast.
		;
		mov	di, offset filename
		mov	al, ':'
		mov	cx, size filename
		repne	scasb
		jne	FS0_7		; No colon -- use filename itself
		mov	copyPosition, di; store position after :
FS0_7:

		;
		; Find the first matching file
		; 
		mov	dx, offset filename
		clr	cx		; Only normal files
		mov	ah, 4eh		; Find first
		int	21h
		
		jnc	FS1
		;
		; No such luck. Tell user the pattern was bad and tell com43
		; to go away.
		; 
		mov	bx, offset noSuchFile
		call	ScrWriteString
		mov	err, 1
		mov	ax, NAK
		call	ComWrite
		jmp	FSDone
FS1:
		;
		; Have a match in dta.fName. Try and open the beast.
		; 
		mov	al, SYNC	; Send Sync byte
		call	ComWrite

		mov	di, copyPosition
		mov	si, offset dta + dtaFName
FS1_5:
		lodsb
		stosb
		tst	al
		jnz	FS1_5

		mov	bx, offset sending
		call	ScrWriteString
		mov	bx, offset filename
		call	ScrWriteString
		mov	al, ' '
		call	ScrWrite

		mov	dx, offset filename
		clr	al		; read-only
		mov	ah, DOS_FILE_OPEN	; Open file and return handle
		int	21h
		jnc	FS2
		;
		; Couldn't open -- report error to user and abort transfer.
		; The error must be pretty serious for the match to have
		; succeeded but we can't open it, unless MS-DOS is brain-
		; damaged...hmmm.
		; 
		mov	bx, offset filename
		call	ScrWriteString
		mov	bx, offset cantOpen
		call	ScrWriteString
		jmp	FSDone
FS2:
		;
		; Now have open handle in AX. Figure out what type of file
		; it is and its size, send the RECV command, and file name.
		; Wait for sync byte, then send size, and transfer the file
		; using the appropriate function.
		;
		mov	bx, ax		; bx <- file handle

		cmp	rawMode, FILE_SEND_RAW
		jne	getFileType

		; if we passed in the RAW mode flag, then don't bother
		; to read through the file, just get the file size
		; using FILE_POS and mov the binary routine into ax

		mov	ah, DOS_FILE_SEEK
		mov	al, SEEK_END
		mov	cx, 0
		mov	dx, 0
		int	21h	; dx:ax = end of file = file size	
		jc	FSDone
		mov	fSize.high, dx
		mov	fSize.low, ax

		; now reposition the file to the beginning
		mov	ah, DOS_FILE_SEEK
		mov	al, SEEK_START
		mov	cx, 0
		mov	dx, 0
		int	21h	; resets file position to the start

		; now set up binary routine
		mov	ax, offset FileSendBinary
		jmp	haveFileType
getFileType:
		call	FileType
		jc	FSDone
haveFileType:
		push	ax		; save transfer function
		push	bx		; and handle

		mov	al, RECV_COMMAND
		call	ComWrite	; Command byte
		
		;
		; Tail of file name w/zero byte
		; 
		mov	si, offset dta + dtaFName
		cld
FS3:
		lodsb
		push	ax
		call	ComWrite
		pop	ax
		tst	al
		jnz	FS3
		
		;
		; Wait for host's response
		; 
		call	Sync
		jnc	FS4
		cmp	al, SYNC
		jne	FS4

		;
		; Clear up stack and close the handle before aborting...
		;
		pop	bx
		pop	ax
		mov	ah, DOS_FILE_CLOSE
		int	21h
		jmp	short FSRet
FS4:
		
		mov	si, offset fSize
		mov	cx, size fSize
		call	ComWriteBlock
		
		pop	bx		; Restore handle
		
		pop	ax		; and transfer function

		call	ax		; Transfer per file type
		jc	FSCloseAndExit
		
		mov	ah, DOS_FILE_CLOSE		; Close
		int	21h
		
		;
		; Issue cr-lf when done sending.
		;
		mov	bx, offset crlf
		call	ScrWriteString

		;
		; See if abort signalled from keyboard
		; 
		test	sysFlags, MASK SF_EXIT or MASK SF_NUKE_FILE
		jnz	FSDone		; Transfer abort/Quit
		
		;
		; Find next matching file
		; 
		mov	ah, 4fh		; Find next
		int	21h
		jc	FSDone
		jmp	FS1		; More to come
FSDone:
		mov	al, END_COMMAND
		call	ComWrite
FSRet:
		and	sysFlags, NOT MASK SF_NUKE_FILE
		ret
FSCloseAndExit:
		mov	ah, DOS_FILE_CLOSE
		int	21h
		jmp	FSDone
		
FileSend	endp
		bugsym	FileSend
	
program		ends
		end

